#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

# pylint: disable=missing-class-docstring,missing-function-docstring,duplicate-code,line-too-long
"""UT99 command set"""
from __future__ import annotations
from typing import Union

from pysimpleini import KeyNotFoundError, SectionNotFoundError

from .core_gamecfg import GameOptions_Factory_Register, OptionType
from .tool_ini import PySimpleINI_GroupKeysInSection
from .common_ut import GameOption_UT


class PySimpleINI_UT99(PySimpleINI_GroupKeysInSection):
    GroupRules = [
        ("XC_Engine.XC_GameEngine", "ServerPackages"),
        ("XC_Engine.XC_GameEngine", "ServerActors"),
        ("Engine.GameEngine", "ServerPackages"),
        ("Engine.GameEngine", "ServerActors"),
        ("Core.System", "Suppress"),
        ("Editor.EditorEngine", "EditPackages"),
    ]


class GameOption_UT99(GameOption_UT):
    szGameType = "ut99"
    Cls_PySimpleINI = PySimpleINI_UT99


class GameOption_UT99_GenAdd(GameOption_UT99):
    bForceAdd = True

    def rem(self, value: Union[None, str]) -> None:
        try:
            super().rem(value)
        except KeyNotFoundError:
            pass

    def set(self, value: str) -> None:
        # force to call the rem method of THIS level
        GameOption_UT99_GenAdd.rem(self, value)
        super().set(value)


class GameOption_UT99_GenAdd__Engine(GameOption_UT99_GenAdd):
    szSectionName = "Engine.GameEngine"
    TValueType = OptionType.OT_STRING
    bRemovable = True

    def set(self, value: str) -> None:
        prev = self.szSectionName

        try:
            self.inifile.getsection("XC_Engine.XC_GameEngine")
            self.szSectionName = "XC_Engine.XC_GameEngine"
            super().set(value)
        except SectionNotFoundError:
            pass

        self.szSectionName = "Engine.GameEngine"
        super().set(value)
        self.szSectionName = prev

    def rem(self, value: Union[None, str]) -> None:
        prev = self.szSectionName

        try:
            self.inifile.getsection("XC_Engine.XC_GameEngine")
            self.szSectionName = "XC_Engine.XC_GameEngine"
            super().rem(value)
        except SectionNotFoundError:
            pass
        self.szSectionName = "Engine.GameEngine"
        super().rem(value)
        self.szSectionName = prev


@GameOptions_Factory_Register
class GameOption_UT99_ServerPackages(GameOption_UT99_GenAdd__Engine):
    szOptionName = "ServerPackages"
    szKeyName = "ServerPackages"
    szHelp = "Add a ServerPackages record"


@GameOptions_Factory_Register
class GameOption_UT99_ServerActors(GameOption_UT99_GenAdd__Engine):
    szOptionName = "ServerActors"
    szKeyName = "ServerActors"
    szHelp = "Add a ServerActors record"


@GameOptions_Factory_Register
class GameOption_UT99_Port(GameOption_UT99):
    szOptionName = "Port"
    szSectionName = "URL"
    szKeyName = "Port"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "7777"
    szHelp = "Server Listening port"


@GameOptions_Factory_Register
class GameOption_UT99_Map(GameOption_UT99):
    szOptionName = "Map"
    szSectionName = "URL"
    szKeyName = "Map"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "dm-deck16]["
    szHelp = "Server Map"


@GameOptions_Factory_Register
class GameOption_UT99_GameType(GameOption_UT99):
    szOptionName = "GameType"
    szSectionName = "Engine.Engine"
    szKeyName = "DefaultServerGame"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "Botpack.DeathMatchPlus"
    szHelp = "Server Gametype"


@GameOptions_Factory_Register
class GameOption_UT99_HostName(GameOption_UT99):
    szOptionName = "HostName"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "ServerName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "ChaCha Test Server"
    szHelp = "Server's HostName"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.GameReplicationInfo", "ShortName", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_MOTD(GameOption_UT99):
    szOptionName = "MOTD"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "MOTDLine1"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "Welcome to ChaCha's server"
    szHelp = "Message of the day"


@GameOptions_Factory_Register
class GameOption_UT99_MOTD2(GameOption_UT99):
    szOptionName = "MOTD2"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "MOTDLine2"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "Enjoy your stay and have fun"
    szHelp = "Message of the day (2)"


@GameOptions_Factory_Register
class GameOption_UT99_MOTD3(GameOption_UT99):
    szOptionName = "MOTD3"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "MOTDLine3"
    TValueType = OptionType.OT_STRING
    szDefaultValue = ""
    szHelp = "Message of the day (3)"


@GameOptions_Factory_Register
class GameOption_UT99_MOTD4(GameOption_UT99):
    szOptionName = "MOTD4"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "MOTDLine4"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "Game Server by ChaCha"
    szHelp = "Message of the day (4)"


@GameOptions_Factory_Register
class GameOption_UT99_AdminEmail(GameOption_UT99):
    szOptionName = "AdminEmail"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "AdminEmail"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "chachacorp@protonmail.com"
    szHelp = "Admin mail"


@GameOptions_Factory_Register
class GameOption_UT99_AdminName(GameOption_UT99):
    szOptionName = "AdminName"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "AdminName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "chacha"
    szHelp = "Admin name"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("UTServerAdmin.UTServerAdmin", "AdminUsername", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_HTTPDownloadServer(GameOption_UT99):
    szOptionName = "HTTPDownloadServer"
    szSectionName = "IpDrv.HTTPDownload"
    szKeyName = "RedirectToURL"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "http://chacha.ddns.net/games/ut99"
    szHelp = "FastDL url"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("IpDrv.HTTPDownload", "UseCompression", "True")
        self.inifile.delkey("IpDrv.HTTPDownload", "ProxyServerHost")
        self.inifile.delkey("IpDrv.HTTPDownload", "ProxyServerPort")
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_MaxClientRate(GameOption_UT99):
    szOptionName = "MaxClientRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "MaxClientRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "25000"
    szHelp = "Max Client Rate"


@GameOptions_Factory_Register
class GameOption_UT99_MinClientRate(GameOption_UT99):
    szOptionName = "MinClientRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "MinClientRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "12000"
    szHelp = "Max Client Rate"


@GameOptions_Factory_Register
class GameOption_UT99_NetServerMaxTickRate(GameOption_UT99):
    szOptionName = "NetServerMaxTickRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "NetServerMaxTickRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "60"
    szHelp = "Server Max TickRate"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.DemoRecDriver", "NetServerMaxTickRate", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_LanServerMaxTickRate(GameOption_UT99):
    szOptionName = "LanServerMaxTickRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "LanServerMaxTickRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "60"
    szHelp = "Lan Server Max TickRate"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.DemoRecDriver", "LanServerMaxTickRate", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_AdminPassword(GameOption_UT99):
    szOptionName = "AdminPassword"
    szSectionName = "Engine.GameInfo"
    szKeyName = "AdminPassword"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "cfographut"
    szHelp = "Admin password"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("UTServerAdmin.UTServerAdmin", "AdminPassword", value)
        self.inifile.writefile()

    def get(self) -> Union[str, list[str]]:
        try:
            return super().get()
        except KeyNotFoundError:
            return self.inifile.getkeyvalue(
                "UTServerAdmin.UTServerAdmin",
                "AdminPassword",
            )

        raise NotImplementedError("method not implemented")


@GameOptions_Factory_Register
class GameOption_UT99_GamePassword(GameOption_UT99):
    szOptionName = "GamePassword"
    szSectionName = "Engine.GameInfo"
    szKeyName = "GamePassword"
    TValueType = OptionType.OT_STRING
    szDefaultValue = ""
    szHelp = "Game password"


@GameOptions_Factory_Register
class GameOption_UT99_MaxPlayers(GameOption_UT99):
    szOptionName = "MaxPlayers"
    szSectionName = "Engine.GameInfo"
    szKeyName = "MaxPlayers"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Max Players"


@GameOptions_Factory_Register
class GameOption_UT99_MaxSpectators(GameOption_UT99):
    szOptionName = "MaxSpectators"
    szSectionName = "Engine.GameInfo"
    szKeyName = "MaxSpectators"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Max Spectators"


@GameOptions_Factory_Register
class GameOption_UT99_AS_TimeLimit(GameOption_UT99):
    szOptionName = "AS_TimeLimit"
    szSectionName = "Botpack.Assault"
    szKeyName = "TimeLimit"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Time Limit"


@GameOptions_Factory_Register
class GameOption_UT99_DOM_TimeLimit(GameOption_UT99):
    szOptionName = "DOM_TimeLimit"
    szSectionName = "Botpack.Domination"
    szKeyName = "TimeLimit"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Time Limit"


@GameOptions_Factory_Register
class GameOption_UT99_CTF_TimeLimit(GameOption_UT99):
    szOptionName = "CTF_TimeLimit"
    szSectionName = "Botpack.CTFGame"
    szKeyName = "TimeLimit"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Time Limit"


@GameOptions_Factory_Register
class GameOption_UT99_DM_TimeLimit(GameOption_UT99):
    szOptionName = "DM_TimeLimit"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "TimeLimit"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "10"
    szHelp = "Game Time Limit"


@GameOptions_Factory_Register
class GameOption_UT99_GoalTeamScore(GameOption_UT99):
    szOptionName = "GoalTeamScore"
    szSectionName = "Botpack.TeamGamePlus"
    szKeyName = "GoalTeamScore"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "3"
    szHelp = "Game Score Limit"


@GameOptions_Factory_Register
class GameOption_UT99_MaxTeamSize(GameOption_UT99):
    szOptionName = "MaxTeamSize"
    szSectionName = "Botpack.TeamGamePlus"
    szKeyName = "MaxTeamSize"
    TValueType = OptionType.OT_FLOAT
    szDefaultValue = "12"
    szHelp = "Maximum Team Size"


@GameOptions_Factory_Register
class GameOption_UT99_DM_FragLimit(GameOption_UT99):
    szOptionName = "DM_FragLimit"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "FragLimit"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "10"
    szHelp = "Game Score Limit"


@GameOptions_Factory_Register
class GameOption_UT99_ServerLogName(GameOption_UT99):
    szOptionName = "ServerLogName"
    szSectionName = "Engine.GameInfo"
    szKeyName = "ServerLogName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "server.log"
    szHelp = "Server Log Name"


@GameOptions_Factory_Register
class GameOption_UT99_WebServer(GameOption_UT99):
    szOptionName = "WebServer"
    szSectionName = "UWeb.WebServer"
    szKeyName = "ListenPort"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "8076"
    szHelp = "enable Web Server"

    def set(self, value: str) -> None:
        # fix ut99 v469c
        try:
            self.inifile.delkey("UWeb.WebServer", "Listenport")
        except KeyNotFoundError:
            pass
        super().set(value)
        if int(value) > 0:
            self.inifile.setaddkeyvalue("UWeb.WebServer", "bEnabled", "True")
        else:
            self.inifile.setaddkeyvalue("UWeb.WebServer", "bEnabled", "False")
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT99_TournamentMode(GameOption_UT99):
    szOptionName = "TournamentMode"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "bTournament"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "False"
    szHelp = "Enable Tournament Mode"


@GameOptions_Factory_Register
class GameOption_UT99_InitialBots(GameOption_UT99):
    szOptionName = "InitialBots"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "InitialBots"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "0"
    szHelp = "Initial Bots number"


@GameOptions_Factory_Register
class GameOption_UT99_MinPlayers(GameOption_UT99):
    szOptionName = "MinPlayers"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "MinPlayers"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "0"
    szHelp = "Minimum player count (allow to adjust bots)"


@GameOptions_Factory_Register
class GameOption_UT99_AS_UseTranslocator(GameOption_UT99):
    szOptionName = "AS_UseTranslocator"
    szSectionName = "Botpack.CTFGame"
    szKeyName = "bUseTranslocator"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Enable Translocator"


@GameOptions_Factory_Register
class GameOption_UT99_CTF_UseTranslocator(GameOption_UT99):
    szOptionName = "CTF_UseTranslocator"
    szSectionName = "Botpack.CTFGame"
    szKeyName = "bUseTranslocator"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Enable Translocator"


@GameOptions_Factory_Register
class GameOption_UT99_DM_UseTranslocator(GameOption_UT99):
    szOptionName = "DM_UseTranslocator"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "bUseTranslocator"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Enable Translocator"


@GameOptions_Factory_Register
class GameOption_UT99_DOM_UseTranslocator(GameOption_UT99):
    szOptionName = "DOM_UseTranslocator"
    szSectionName = "Botpack.Domination"
    szKeyName = "bUseTranslocator"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Enable Translocator"


@GameOptions_Factory_Register
class GameOption_UT99_CTF_ForceRespawn(GameOption_UT99):
    szOptionName = "CTF_ForceRespawn"
    szSectionName = "Botpack.CTFGame"
    szKeyName = "bForceRespawn"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Force Player to respawn"


@GameOptions_Factory_Register
class GameOption_UT99_DM_ForceRespawn(GameOption_UT99):
    szOptionName = "DM_ForceRespawn"
    szSectionName = "Botpack.DeathMatchPlus"
    szKeyName = "bForceRespawn"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Force Player to respawn"


@GameOptions_Factory_Register
class GameOption_UT99_DOM_ForceRespawn(GameOption_UT99):
    szOptionName = "DOM_ForceRespawn"
    szSectionName = "Botpack.Domination"
    szKeyName = "bForceRespawn"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Force Player to respawn"


@GameOptions_Factory_Register
class GameOption_UT99_AS_ForceRespawn(GameOption_UT99):
    szOptionName = "AS_ForceRespawn"
    szSectionName = "Botpack.Assault"
    szKeyName = "bForceRespawn"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "True"
    szHelp = "Force Player to respawn"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

# pylint: disable=missing-class-docstring,missing-module-docstring,missing-function-docstring,duplicate-code,line-too-long
from __future__ import annotations
from typing import Union

import re
from os.path import join
from os import linesep

from .core_gamecfg import GameOptions_Factory_Register, GameOption, OptionType, GameCfgException


class COD4KeyNotFoundError(GameCfgException):
    """Exception to catch non existing configuration key in COD4 configuration file"""


class GameOption_COD4(GameOption):
    szGameType = "cod4"
    TValueType = OptionType.OT_INVALID

    szOptionName: str = ""
    szKeyName: str = ""
    bDblQuoted: bool = False
    szPrefix: str = ""

    def __init__(self, GameRootDir: str, ConfigFileRelPath: str) -> None:
        super().__init__(GameRootDir, ConfigFileRelPath)
        self.mainConfigFilePath = join(GameRootDir, ConfigFileRelPath)
        self.cfgfile = open(self.mainConfigFilePath, "r", encoding="utf8")  # pylint: disable=consider-using-with

    def format_OT_BOOLEAN(self, value: Union[int, str, float]) -> str:
        return "1" if super().format_OT_BOOLEAN(value).lower() == "true" else "0"

    def close(self) -> None:
        self.cfgfile.close()

    def set(self, value: str) -> None:
        if not self.szOptionName:
            raise RuntimeError("szOptionName is not set")

        FinalValue: str = value
        if self.bDblQuoted:
            FinalValue = '"' + FinalValue + '"'
        if self.szPrefix != "":
            FinalValue = self.szPrefix + " " + self.szKeyName + " " + FinalValue + linesep
        bfound = False
        newFile = ""

        regex = r"^\s*" + self.szPrefix + r"\s+" + self.szKeyName + r"\s*(?P<value>.*)"

        for line in self.cfgfile.readlines():
            if re.search(regex, line):
                if bfound:
                    print("[warning] Option defined multiple time")
                newFile += FinalValue
                bfound = True
            else:
                newFile += line

        if not bfound:
            # write new key at the top of the file to be sure we do not erase the map / map_rotate cmd
            newFile = FinalValue + newFile

        self.cfgfile.close()
        with open(self.mainConfigFilePath, "w", encoding="utf8") as ofile:
            ofile.write(newFile)
        self.cfgfile = open(self.mainConfigFilePath, "r", encoding="utf8")  # pylint: disable=consider-using-with

    def rem(self, value: Union[str, None] = None) -> None:
        if not self.szOptionName:
            raise RuntimeError("szOptionName is not set")

        regex = r"^\s*" + self.szPrefix + r"\s+" + self.szKeyName + r"\s*(?P<value>.*)"

        bfound = False
        newFile = ""
        for line in self.cfgfile.readlines():
            if re.search(regex, line):
                if bfound:
                    print("[warning] Option defined multiple time")
                bfound = True
            else:
                newFile += line
        if not bfound:
            raise COD4KeyNotFoundError("Option not found in file")

        self.cfgfile.close()
        with open(self.mainConfigFilePath, "w", encoding="utf8") as ofile:
            ofile.write(newFile)
        self.cfgfile = open(self.mainConfigFilePath, "r", encoding="utf8")  # pylint: disable=consider-using-with

    def get(self) -> str:
        if not self.szOptionName:
            raise RuntimeError("szOptionName not set")

        if self.bDblQuoted:
            regex = r"^\s*" + self.szPrefix + r"\s+" + self.szKeyName + r"\s*\"(?P<value>.*)\""
        else:
            regex = r"^\s*" + self.szPrefix + r"\s+" + self.szKeyName + r"\s*(?P<value>.*)"
        bfound = False

        for line in self.cfgfile.readlines():
            if result := re.search(regex, line):
                if bfound:
                    raise RuntimeError("Option defined multiple time")
                res = result.groupdict()["value"]
                bfound = True
        if not bfound:
            raise COD4KeyNotFoundError("Option not found in file")
        return res


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Admin(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_admin"
    szKeyName: str = "_Admin"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = "ServAdmin"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Email(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_email"
    szKeyName: str = "_Email"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = "ServEmail"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Website(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_website"
    szKeyName: str = "_Website"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = "ServWebsite"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Location(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_location"
    szKeyName: str = "_Location"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = "ServLocation"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Maps(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_maps"
    szKeyName: str = "_Maps"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = ""
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Meta_Gametype(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "meta_gametype"
    szKeyName: str = "_Gametype"
    bDblQuoted: bool = True
    szPrefix = "sets"
    szDefaultValue = ""
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_Cod4x_AuthToken(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "cod4x_authtoken"
    szKeyName: str = "sv_authtoken"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = "Cod4x personnal auth token"


@GameOptions_Factory_Register
class GameOption_COD4_Hostname(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "hostname"
    szKeyName: str = "sv_hostname"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = "Server Hostname"


@GameOptions_Factory_Register
class GameOption_COD4_MOTD(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "motd"
    szKeyName: str = "g_motd"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "Welcome"
    szHelp = "Server Message Of The Day"


@GameOptions_Factory_Register
class GameOption_COD4_dedicated(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dedicated"
    szKeyName: str = "dedicated"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "2"
    szHelp = """0 = Listen, 1 = LAN, 2 = Internet ( you probably want 2 )"""


@GameOptions_Factory_Register
class GameOption_COD4_RCON_password(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "rcon_password"
    szKeyName: str = "rcon_password"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = "password for remote access, leave empty to deactivate, min 8 characters"


@GameOptions_Factory_Register
class GameOption_COD4_game_password(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "game_password"
    szKeyName: str = "g_password"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = "join password, leave empty to deactivate"


@GameOptions_Factory_Register
class GameOption_COD4_nb_private_clients(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "nb_privateClients"
    szKeyName: str = "sv_privateClients"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "3"
    szHelp = """Private Clients, number of slots that can only be changed with a password"""


@GameOptions_Factory_Register
class GameOption_COD4_private_password(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "private_password"
    szKeyName: str = "sv_privatePassword"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = "the password to join private slots"


@GameOptions_Factory_Register
class GameOption_COD4_maxclients(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "maxclients"
    szKeyName: str = "sv_maxclients"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "12"
    szHelp = """Maximum client number"""


@GameOptions_Factory_Register
class GameOption_COD4_logsync(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "logsync"
    szKeyName: str = "g_logsync"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "2"
    szHelp = """0=no log, 1=buffered, 2=continuous, 3=append"""


@GameOptions_Factory_Register
class GameOption_COD4_enable_logfile(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "enable_logfile"
    szKeyName: str = "logfile"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """0 = NO log, 1 = log file enabled"""


@GameOptions_Factory_Register
class GameOption_COD4_logfile(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "logfile"
    szKeyName: str = "g_log"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "games_mp.log"
    szHelp = "Name of log file, default is games_mp.log"


@GameOptions_Factory_Register
class GameOption_COD4_enable_logdamage(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "enable_logdamage"
    szKeyName: str = "sv_log_damage"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """0 = NO logdamage, 1 = logdamage enabled"""


@GameOptions_Factory_Register
class GameOption_COD4_statusfile(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "statusfile"
    szKeyName: str = "sv_statusfile"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "serverstatus.xml"
    szHelp = "writes an xml serverstatus to disc, leave empty to disable"


@GameOptions_Factory_Register
class GameOption_COD4_ney_port(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "net_port"
    szKeyName: str = "net_port"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "28960"
    szHelp = """network port"""


@GameOptions_Factory_Register
class GameOption_COD4_maxRate(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "maxRate"
    szKeyName: str = "sv_maxRate"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "25000"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_minPing(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "minPing"
    szKeyName: str = "sv_minPing"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = "minimal ping [ms] for a player to join the server"


@GameOptions_Factory_Register
class GameOption_COD4_maxPing(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "maxPing"
    szKeyName: str = "sv_maxPing"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "350"
    szHelp = "maximal ping [ms] for a player to join the server"


@GameOptions_Factory_Register
class GameOption_COD4_randomMapRotation(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "randomMapRotation"
    szKeyName: str = "sv_randomMapRotation"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """0 = sv_mapRotation is randomized, 1 = sequential order of sv_mapRotation"""


@GameOptions_Factory_Register
class GameOption_COD4_teambalance(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "teambalance"
    szKeyName: str = "scr_teambalance"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = "auto-teambalance //0 = no, 1 = yes"


@GameOptions_Factory_Register
class GameOption_COD4_team_fftype(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "team_fftype"
    szKeyName: str = "scr_team_fftype"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = "friendly-fire //0 = off, 1 = on, //2 = reflect damage, 3 = shared damage"


@GameOptions_Factory_Register
class GameOption_COD4_enable_hardcore(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "hardcore"
    szKeyName: str = "scr_hardcore"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """Hardcore Mode //0 = off 1 = on"""


@GameOptions_Factory_Register
class GameOption_COD4_enable_oldschool(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "oldschool"
    szKeyName: str = "scr_oldschool"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """Oldschool Mode //0 = off, 1 = on"""


@GameOptions_Factory_Register
class GameOption_COD4_enable_friendlyPlayerCanBlock(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "friendlyPlayerCanBlock"
    szKeyName: str = "g_friendlyPlayerCanBlock"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """1 = player collision between friendly players, 0 = collision between friendly players is disabled"""


@GameOptions_Factory_Register
class GameOption_COD4_enable_FFAPlayerCanBlock(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "FFAPlayerCanBlock"
    szKeyName: str = "g_FFAPlayerCanBlock"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """1 = player collision, 0 = collision between players is disabled"""


@GameOptions_Factory_Register
class GameOption_COD4_DM_scorelimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_scorelimit"
    szKeyName: str = "scr_dm_scorelimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1000"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_DM_timelimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_timelimit"
    szKeyName: str = "scr_dm_timelimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "15"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_DM_roundlimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_roundlimit"
    szKeyName: str = "scr_dm_roundlimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_DM_numlives(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_numlives"
    szKeyName: str = "scr_dm_numlives"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_DM_playerrespawndelay(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_playerrespawndelay"
    szKeyName: str = "scr_dm_playerrespawndelay"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_DM_waverespawndelay(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "dm_waverespawndelay"
    szKeyName: str = "scr_dm_waverespawndelay"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_scorelimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_scorelimit"
    szKeyName: str = "scr_war_scorelimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "2000"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_timelimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_timelimit"
    szKeyName: str = "scr_war_timelimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "10"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_roundlimit(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_roundlimit"
    szKeyName: str = "scr_war_roundlimit"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_numlives(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_numlives"
    szKeyName: str = "scr_war_numlives"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_playerrespawndelay(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_playerrespawndelay"
    szKeyName: str = "scr_war_playerrespawndelay"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_TDM_waverespawndelay(GameOption_COD4):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "tdm_waverespawndelay"
    szKeyName: str = "scr_war_waverespawndelay"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_gametype(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "gametype"
    szKeyName: str = "g_gametype"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "dm"
    szHelp = "gamemode, one of [war, dm, sd, sab, koth]"


@GameOptions_Factory_Register
class GameOption_COD4_mapRotation(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mapRotation"
    szKeyName: str = "sv_mapRotation"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "gametype dm map mp_block"
    szHelp = """Map rotation list"""


@GameOptions_Factory_Register
class GameOption_COD4_allowdownloadk(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "allowdownload"
    szKeyName: str = "sv_allowdownload"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_wwwDownload(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "wwwDownload"
    szKeyName: str = "sv_wwwDownload"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_wwwBaseURL(GameOption_COD4):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "wwwBaseURL"
    szKeyName: str = "sv_wwwBaseURL"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_wwwDlDisconnected(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "wwwDlDisconnected"
    szKeyName: str = "sv_wwwDlDisconnected"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = ""


@GameOptions_Factory_Register
class GameOption_COD4_nosteamnames(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "nosteamnames"
    szKeyName: str = "sv_nosteamnames"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = "1 = Use names from steam if steam is available"


@GameOptions_Factory_Register
class GameOption_COD4_punkbuster(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "punkbuster"
    szKeyName: str = "sv_punkbuster"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = "Enable Punkbuste (PB is not supported on CoD4x)"


@GameOptions_Factory_Register
class GameOption_COD4_pure(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "pure"
    szKeyName: str = "sv_pure"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = "check IWD-data 0 = off, 1 = on"


@GameOptions_Factory_Register
class GameOption_COD4_antilag(GameOption_COD4):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "antilag"
    szKeyName: str = "g_antilag"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = "0 = off, 1 = on // Anti lag checks for weapon hits"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

"""utility module that contain PySimpleINI based helpers"""
from __future__ import annotations
from typing import Union
from pysimpleini import PySimpleINI, SectionNotFoundError, Key, Section


class PySimpleINI_GroupKeysInSection(PySimpleINI):
    """a class base on PySimpleINI that allow user to force some key to be group together in a section"""

    GroupRules: list[tuple[str, str]] = []

    def groupkeysinsection(self, szSectionName: str, szKeyName: str) -> None:
        """internal function that actually group keys"""
        try:
            _section: list[Section] = self.getsection(szSectionName)
            if len(_section) == 1:
                ar_ServerPackages: Union[list[Key], Key] = _section[0].getkey(szKeyName)
                if isinstance(ar_ServerPackages, Key):
                    ar_ServerPackages = [ar_ServerPackages]
                else:  # array
                    pass
                for ServerPackages in ar_ServerPackages:
                    _section[0].delkey(ServerPackages.getname(), None, ServerPackages.getvalue())
                for ServerPackages in ar_ServerPackages:
                    _section[0].setaddkeyvalue(ServerPackages.getname(), ServerPackages.getvalue(), True)
        except SectionNotFoundError:
            pass

    def writefile(self, bBeautify: bool = False, bWipeComments: bool = False) -> None:
        """overload of the write function to call the group function before"""
        for GroupRule in self.GroupRules:
            self.groupkeysinsection(GroupRule[0], GroupRule[1])
        super().writefile(bBeautify, bWipeComments)



# pygamecfg (c) by chacha
#
# pygamecfg  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

import unittest
from os import path, chdir
from io import StringIO
from contextlib import redirect_stdout, redirect_stderr
from pathlib import Path
import shutil

from src import pygamecfg
from src.pygamecfg.__main__ import fct_main
from src.pygamecfg.game_cod4 import COD4KeyNotFoundError

testdir_path = Path(__file__).parent.resolve()


class Testtest_cod4(unittest.TestCase):
    def setUp(self) -> None:
        chdir(testdir_path.parent.resolve())
        self.CleanTmp()
        shutil.copytree(testdir_path / "data", testdir_path / "tmp")
        print("======================")

    def CleanTmp(self):
        # remove any file in tmp dir, except .keep
        if path.exists(testdir_path / "tmp"):
            shutil.rmtree(testdir_path / "tmp")

    def test_normal_READ_sv_maxclients(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "maxclients"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("11\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_READ_sv_mapRotation(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "mapRotation"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual(
                "gametype dm map mp_backlot gametype dm map mp_bloc gametype dm map mp_bog gametype dm map mp_cargoship gametype dm map mp_citystreets gametype dm map mp_convoy gametype dm map mp_countdown gametype dm map mp_crash gametype dm map mp_crossfire gametype dm map mp_farm gametype dm map mp_overgrown gametype dm map mp_pipeline gametype dm map mp_shipment gametype dm map mp_showdown gametype dm map mp_strike gametype dm map mp_vacant\n",
                capted_stdout.getvalue(),
            )
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_WRITE_sv_maxclients(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "SetOption", "maxclients", "17"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("setting option <maxclients>  to:  17\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "maxclients"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("17\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())
        # check if *other* key still there / untouched
        self.test_normal_READ_sv_mapRotation()

    def test_defect_READ_net_port_NONEXISTS(self):
        with self.assertRaises(COD4KeyNotFoundError):
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "net_port"])

    def test_normal_WRITE_net_port_NONEXISTS(self):
        fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "SetOption", "net_port", "132"])
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "net_port"])
            self.assertEqual("132\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_WRITE_oldschool(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "SetOption", "oldschool", "1"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("setting option <oldschool>  to:  1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "oldschool"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "SetOption", "oldschool", "0"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("setting option <oldschool>  to:  0\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "oldschool"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("0\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "SetOption", "oldschool", "1"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("setting option <oldschool>  to:  1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "cod4", "-b", "test/tmp/COD4", "GetOption", "oldschool"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

# pylint: disable=missing-class-docstring,missing-module-docstring,missing-function-docstring,duplicate-code,line-too-long

from __future__ import annotations

from .core_gamecfg import GameOptions_Factory_Register, OptionType
from .game_cod4 import GameOption_COD4


class GameOption_COD4_ProMod(GameOption_COD4):
    szGameType = "cod4_promod"


@GameOptions_Factory_Register
class GameOption_COD4_ProMod_promod_mode(GameOption_COD4_ProMod):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "promod_mode"
    szKeyName: str = "promod_mode"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "comp_public"
    szHelp = """comp_public,custom_public,2v2_mr,1v1_mr,knockout_mr,match_mr"""


@GameOptions_Factory_Register
class GameOption_COD4_ProMod_promod_enable_scorebot(GameOption_COD4_ProMod):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "promod_enable_scorebot"
    szKeyName: str = "promod_enable_scorebot"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """match-modes only"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

"""common UT functions"""
from __future__ import annotations
from typing import Union

from os.path import join
from pathlib import Path

from pysimpleini import PySimpleINI

from .core_gamecfg import GameOption, OptionType


class GameOption_UT(GameOption):
    """generic UT Option class"""

    szGameType = ""
    TValueType = OptionType.OT_INVALID

    szOptionName = ""
    szSectionName = ""
    szKeyName = ""
    bForceAdd: bool = False
    bRemovable: bool = False

    cachedFile: Union[None, PySimpleINI] = None
    cachedFilePath: Union[None, Path] = None

    Cls_PySimpleINI: type[PySimpleINI] = PySimpleINI

    @classmethod
    def openFile(cls, filepath: Path) -> PySimpleINI:
        """Open the file"""
        if (not cls.cachedFile) or (filepath != cls.cachedFilePath):
            cls.cachedFilePath = filepath
            cls.cachedFile = cls.Cls_PySimpleINI(filepath)
        return cls.cachedFile

    def __init__(self, GameRootDir: str, ConfigFileRelPath: str) -> None:
        super().__init__(GameRootDir, ConfigFileRelPath)
        self.mainConfigFilePath: Path = Path(join(GameRootDir, ConfigFileRelPath))
        self.inifile = self.openFile(self.mainConfigFilePath)

    def set(self, value: str) -> None:
        if not self.szOptionName:
            raise RuntimeError("szOptionName is not set")
        self.format(value)
        self.inifile.setaddkeyvalue(self.szSectionName, self.szKeyName, value, self.bForceAdd)
        self.inifile.writefile()

    def rem(self, value: Union[None, str]) -> None:
        if not self.szOptionName:
            raise RuntimeError("szOptionName is not set")
        if not self.bRemovable:
            raise RuntimeError("this options is not removable")
        self.inifile.delkey_ex(self.szSectionName, self.szKeyName, None, value)
        self.inifile.writefile()

    def get(self) -> Union[str, list[str]]:
        if not self.szOptionName:
            raise RuntimeError("szOptionName not set")
        return self.inifile.getkeyvalue(self.szSectionName, self.szKeyName)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

# pylint: disable=missing-class-docstring,missing-function-docstring,duplicate-code,line-too-long
"""UT2k4 command set"""
from __future__ import annotations
from typing import Union

from pysimpleini import KeyNotFoundError

from .core_gamecfg import GameOptions_Factory_Register, OptionType
from .tool_ini import PySimpleINI_GroupKeysInSection
from .common_ut import GameOption_UT


class PySimpleINI_UT2k4(PySimpleINI_GroupKeysInSection):
    GroupRules = [
        ("Engine.GameEngine", "ServerPackages"),
        ("Engine.GameEngine", "ServerActors"),
        ("Core.System", "Suppress"),
        ("Core.System", "Paths"),
        ("Core.System", "Paths"),
        ("Editor.EditorEngine", "EditPackages"),
        ("Editor.EditorEngine", "CutdownPackages"),
    ]


class GameOption_UT2k4(GameOption_UT):
    szGameType = "ut2k4"
    Cls_PySimpleINI = PySimpleINI_UT2k4


class GameOption_UT2k4_GenAdd(GameOption_UT2k4):
    bForceAdd = True

    def rem(self, value: Union[None, str]) -> None:
        try:
            super().rem(value)
        except KeyNotFoundError:
            pass

    def set(self, value: str) -> None:
        # force to call the rem method of THIS level
        GameOption_UT2k4_GenAdd.rem(self, value)
        super().set(value)


class GameOption_UT2k4_GenAdd__Engine(GameOption_UT2k4_GenAdd):
    szSectionName = "Engine.GameEngine"
    TValueType = OptionType.OT_STRING
    bRemovable = True


@GameOptions_Factory_Register
class GameOption_UT2k4_ServerPackages(GameOption_UT2k4_GenAdd__Engine):
    szOptionName = "ServerPackages"
    szKeyName = "ServerPackages"
    szHelp = "Add a ServerPackages record"


@GameOptions_Factory_Register
class GameOption_UT2k4_ServerActors(GameOption_UT2k4_GenAdd__Engine):
    szOptionName = "ServerActors"
    szKeyName = "ServerActors"
    szHelp = "Add a ServerActors record"


@GameOptions_Factory_Register
class GameOption_UT2k4_MasterServerList(GameOption_UT2k4_GenAdd):
    szSectionName = "IpDrv.MasterServerLink"
    TValueType = OptionType.OT_STRING
    bRemovable = True
    szOptionName = "MasterServerList"
    szKeyName = "MasterServerList"
    szHelp = "Add a MasterServerList record"


@GameOptions_Factory_Register
class GameOption_UT2k4_Port(GameOption_UT2k4):
    szOptionName = "Port"
    szSectionName = "URL"
    szKeyName = "Port"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "7777"
    szHelp = "Server Listening port"


@GameOptions_Factory_Register
class GameOption_UT2k4_Map(GameOption_UT2k4):
    szOptionName = "Map"
    szSectionName = "URL"
    szKeyName = "Map"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "dm-deck16]["
    szHelp = "Server Map"


@GameOptions_Factory_Register
class GameOption_UT2k4_GameType(GameOption_UT2k4):
    szOptionName = "GameType"
    szSectionName = "Engine.Engine"
    szKeyName = "DefaultServerGame"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "XGame.XDeathmatch"
    szHelp = "Server Gametype"


@GameOptions_Factory_Register
class GameOption_UT2k4_HostName(GameOption_UT2k4):
    szOptionName = "HostName"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "ServerName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "ChaCha Test Server"
    szHelp = "Server's HostName"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.GameReplicationInfo", "ShortName", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT2k4_MOTD(GameOption_UT2k4):
    szOptionName = "MOTD"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "MessageOfTheDay"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "Welcome to ChaCha's server"
    szHelp = "Message of the day"


@GameOptions_Factory_Register
class GameOption_UT2k4_AdminEmail(GameOption_UT2k4):
    szOptionName = "AdminEmail"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "AdminEmail"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "chachacorp@protonmail.com"
    szHelp = "Admin mail"


@GameOptions_Factory_Register
class GameOption_UT2k4_AdminName(GameOption_UT2k4):
    szOptionName = "AdminName"
    szSectionName = "Engine.GameReplicationInfo"
    szKeyName = "AdminName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "chacha"
    szHelp = "Admin name"


@GameOptions_Factory_Register
class GameOption_UT2k4_HTTPDownloadServer(GameOption_UT2k4):
    szOptionName = "HTTPDownloadServer"
    szSectionName = "IpDrv.HTTPDownload"
    szKeyName = "RedirectToURL"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "http://chacha.ddns.net/games/ut2k4"
    szHelp = "FastDL url"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("IpDrv.HTTPDownload", "UseCompression", "True")
        self.inifile.delkey("IpDrv.HTTPDownload", "ProxyServerHost")
        self.inifile.delkey("IpDrv.HTTPDownload", "ProxyServerPort")
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT2k4_MaxClientRate(GameOption_UT2k4):
    szOptionName = "MaxClientRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "MaxClientRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "25000"
    szHelp = "Max Client Rate"


@GameOptions_Factory_Register
class GameOption_UT2k4_MaxInternetClientRate(GameOption_UT2k4):
    szOptionName = "MaxInternetClientRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "MaxClientRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "25000"
    szHelp = "Max Client Rate"


@GameOptions_Factory_Register
class GameOption_UT2k4_NetServerMaxTickRate(GameOption_UT2k4):
    szOptionName = "NetServerMaxTickRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "NetServerMaxTickRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "60"
    szHelp = "Server Max TickRate"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.DemoRecDrive", "NetServerMaxTickRate", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT2k4_LanServerMaxTickRate(GameOption_UT2k4):
    szOptionName = "LanServerMaxTickRate"
    szSectionName = "IpDrv.TcpNetDriver"
    szKeyName = "LanServerMaxTickRate"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "60"
    szHelp = "Lan Server Max TickRate"

    def set(self, value: str) -> None:
        super().set(value)
        self.inifile.setaddkeyvalue("Engine.DemoRecDrive", "LanServerMaxTickRate", value)
        self.inifile.writefile()


@GameOptions_Factory_Register
class GameOption_UT2k4_AdminPassword(GameOption_UT2k4):
    szOptionName = "AdminPassword"
    szSectionName = "Engine.AccessControl"
    szKeyName = "AdminPassword"
    TValueType = OptionType.OT_STRING
    szDefaultValue = "cfographut"
    szHelp = "Admin password"


@GameOptions_Factory_Register
class GameOption_UT2k4_GamePassword(GameOption_UT2k4):
    szOptionName = "GamePassword"
    szSectionName = "Engine.AccessControl"
    szKeyName = "GamePassword"
    TValueType = OptionType.OT_STRING
    szDefaultValue = ""
    szHelp = "Game password"


@GameOptions_Factory_Register
class GameOption_UT2k4_MaxPlayers(GameOption_UT2k4):
    szOptionName = "MaxPlayers"
    szSectionName = "Engine.GameInfo"
    szKeyName = "MaxPlayers"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Max Players"


@GameOptions_Factory_Register
class GameOption_UT2k4_MaxSpectators(GameOption_UT2k4):
    szOptionName = "MaxSpectators"
    szSectionName = "Engine.GameInfo"
    szKeyName = "MaxSpectators"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "20"
    szHelp = "Game Max Spectators"


##############
## TimeLimit
#############
class GameOption_UT2k4_TimeLimit(GameOption_UT2k4):
    szOptionName = "TimeLimit"
    szKeyName = "TimeLimit"
    TValueType = OptionType.OT_INTEGER
    szHelp = "Game Time Limit"


@GameOptions_Factory_Register
class GameOption_UT2k4_CTF_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "CTF_TimeLimit"
    szSectionName = "xGame.xCTFGame"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_DM_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "DM_TimeLimit"
    szSectionName = "XGame.xDeathMatch"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_TDM_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "TDM_TimeLimit"
    szSectionName = "XGame.xTeamGame"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_ONS_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "ONS_TimeLimit"
    szSectionName = "Onslaught.ONSOnslaughtGame"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_DOM_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "DOM_TimeLimit"
    szSectionName = "XGame.xDoubleDom"
    szDefaultValue = "3"


@GameOptions_Factory_Register
class GameOption_UT2k4_BR_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "BR_TimeLimit"
    szSectionName = "XGame.xBombingRun"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_AS_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "AS_TimeLimit"
    szSectionName = "UT2k4Assault.ASGameInfo"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_LMS_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "LMS_TimeLimit"
    szSectionName = "BonusPack.xLastManStandingGame"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_INV_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "INV_TimeLimit"
    szSectionName = "SkaarjPack.Invasion"
    szDefaultValue = "20"


@GameOptions_Factory_Register
class GameOption_UT2k4_MUT_TimeLimit(GameOption_UT2k4_TimeLimit):
    szOptionName = "MUT_TimeLimit"
    szSectionName = "BonusPack.xMutantGame"
    szDefaultValue = "20"


##############
## GoalScore
#############
class GameOption_UT2k4_GoalScore(GameOption_UT2k4):
    szOptionName = "GoalScore"
    szKeyName = "GoalScore"
    TValueType = OptionType.OT_INTEGER
    szHelp = "Game Score Goal"


@GameOptions_Factory_Register
class GameOption_UT2k4_CTF_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "CTF_GoalScore"
    szSectionName = "xGame.xCTFGame"
    szDefaultValue = "5"


@GameOptions_Factory_Register
class GameOption_UT2k4_DM_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "DM_GoalScore"
    szSectionName = "XGame.xDeathMatch"
    szDefaultValue = "25"


@GameOptions_Factory_Register
class GameOption_UT2k4_TDM_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "TDM_GoalScore"
    szSectionName = "XGame.xTeamGame"
    szDefaultValue = "60"


@GameOptions_Factory_Register
class GameOption_UT2k4_ONS_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "ONS_GoalScore"
    szSectionName = "Onslaught.ONSOnslaughtGame"
    szDefaultValue = "3"


@GameOptions_Factory_Register
class GameOption_UT2k4_DOM_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "DOM_GoalScore"
    szSectionName = "XGame.xDoubleDom"
    szDefaultValue = "3"


@GameOptions_Factory_Register
class GameOption_UT2k4_BR_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "BR_GoalScore"
    szSectionName = "XGame.xBombingRun"
    szDefaultValue = "15"


@GameOptions_Factory_Register
class GameOption_UT2k4_AS_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "AS_GoalScore"
    szSectionName = "UT2k4Assault.ASGameInfo"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_LMS_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "LMS_GoalScore"
    szSectionName = "BonusPack.xLastManStandingGame"
    szDefaultValue = "25"


@GameOptions_Factory_Register
class GameOption_UT2k4_INV_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "INV_GoalScore"
    szSectionName = "SkaarjPack.Invasion"
    szDefaultValue = "60"


@GameOptions_Factory_Register
class GameOption_UT2k4_MUT_GoalScore(GameOption_UT2k4_GoalScore):
    szOptionName = "MUT_GoalScore"
    szSectionName = "BonusPack.xMutantGame"
    szDefaultValue = "20"


##############
## ForceRespawn
#############
class GameOption_UT2k4_ForceRespawn(GameOption_UT2k4):
    szOptionName = "ForceRespawn"
    szKeyName = "bForceRespawn"
    TValueType = OptionType.OT_BOOLEAN
    szHelp = "Force player to Respawn"


@GameOptions_Factory_Register
class GameOption_UT2k4_CTF_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "CTF_ForceRespawn"
    szSectionName = "xGame.xCTFGame"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_DM_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "DM_ForceRespawn"
    szSectionName = "XGame.xDeathMatch"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_TDM_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "TDM_ForceRespawn"
    szSectionName = "XGame.xTeamGame"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_ONS_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "ONS_ForceRespawn"
    szSectionName = "Onslaught.ONSOnslaughtGame"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_DOM_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "DOM_ForceRespawn"
    szSectionName = "XGame.xDoubleDom"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_BR_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "BR_ForceRespawn"
    szSectionName = "XGame.xBombingRun"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_AS_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "AS_ForceRespawn"
    szSectionName = "UT2k4Assault.ASGameInfo"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_LMS_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "LMS_ForceRespawn"
    szSectionName = "BonusPack.xLastManStandingGame"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_INV_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "INV_ForceRespawn"
    szSectionName = "SkaarjPack.Invasion"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_MUT_ForceRespawn(GameOption_UT2k4_ForceRespawn):
    szOptionName = "MUT_ForceRespawn"
    szSectionName = "BonusPack.xMutantGame"
    szDefaultValue = "True"


##############
## MaxLives
#############
class GameOption_UT2k4_MaxLives(GameOption_UT2k4):
    szOptionName = "MaxLives"
    szKeyName = "MaxLives"
    TValueType = OptionType.OT_INTEGER
    szHelp = "Maximum player lives"


@GameOptions_Factory_Register
class GameOption_UT2k4_CTF_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "CTF_MaxLives"
    szSectionName = "xGame.xCTFGame"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_DM_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "DM_MaxLives"
    szSectionName = "XGame.xDeathMatch"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_TDM_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "TDM_MaxLives"
    szSectionName = "XGame.xTeamGame"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_ONS_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "ONS_MaxLives"
    szSectionName = "Onslaught.ONSOnslaughtGame"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_DOM_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "DOM_MaxLives"
    szSectionName = "XGame.xDoubleDom"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_BR_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "BR_MaxLives"
    szSectionName = "XGame.xBombingRun"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_AS_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "AS_MaxLives"
    szSectionName = "UT2k4Assault.ASGameInfo"
    szDefaultValue = "0"


@GameOptions_Factory_Register
class GameOption_UT2k4_LMS_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "LMS_MaxLives"
    szSectionName = "BonusPack.xLastManStandingGame"
    szDefaultValue = "4"


@GameOptions_Factory_Register
class GameOption_UT2k4_INV_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "INV_MaxLives"
    szSectionName = "SkaarjPack.Invasion"
    szDefaultValue = "4"


@GameOptions_Factory_Register
class GameOption_UT2k4_MUT_MaxLives(GameOption_UT2k4_MaxLives):
    szOptionName = "MUT_MaxLives"
    szSectionName = "BonusPack.xMutantGame"
    szDefaultValue = "0"


##############
## AllowTrans
#############
class GameOption_UT2k4_AllowTrans(GameOption_UT2k4):
    szOptionName = "AllowTrans"
    szKeyName = "bAllowTrans"
    TValueType = OptionType.OT_BOOLEAN
    szHelp = "Maximum player lives"


@GameOptions_Factory_Register
class GameOption_UT2k4_CTF_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "CTF_AllowTrans"
    szSectionName = "xGame.xCTFGame"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_DM_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "DM_AllowTrans"
    szSectionName = "XGame.xDeathMatch"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_TDM_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "TDM_AllowTrans"
    szSectionName = "XGame.xTeamGame"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_ONS_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "ONS_AllowTrans"
    szSectionName = "Onslaught.ONSOnslaughtGame"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_DOM_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "DOM_AllowTrans"
    szSectionName = "XGame.xDoubleDom"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_BR_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "BR_AllowTrans"
    szSectionName = "XGame.xBombingRun"
    szDefaultValue = "True"


@GameOptions_Factory_Register
class GameOption_UT2k4_AS_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "AS_AllowTrans"
    szSectionName = "UT2k4Assault.ASGameInfo"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_LMS_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "LMS_AllowTrans"
    szSectionName = "BonusPack.xLastManStandingGame"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_INV_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "INV_AllowTrans"
    szSectionName = "SkaarjPack.Invasion"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_MUT_AllowTrans(GameOption_UT2k4_AllowTrans):
    szOptionName = "MUT_AllowTrans"
    szSectionName = "BonusPack.xMutantGame"
    szDefaultValue = "False"


@GameOptions_Factory_Register
class GameOption_UT2k4_WebServer_ServerName(GameOption_UT2k4):
    szOptionName = "WebServer_ServerName"
    szSectionName = "UWeb.WebServer"
    szKeyName = "ServerName"
    TValueType = OptionType.OT_STRING
    szDefaultValue = ""
    szHelp = "Server external IP or Name"


@GameOptions_Factory_Register
class GameOption_UT2k4_ServerBehindNAT(GameOption_UT2k4):
    szOptionName = "ServerBehindNAT"
    szSectionName = "IpDrv.MasterServerUplink"
    szKeyName = "ServerBehindNAT"
    TValueType = OptionType.OT_BOOLEAN
    szDefaultValue = "False"
    szHelp = "Is server behind a NAS ?"


@GameOptions_Factory_Register
class GameOption_UT2k4_WebServer(GameOption_UT2k4):
    szOptionName = "WebServer"
    szSectionName = "UWeb.WebServer"
    szKeyName = "ListenPort"
    TValueType = OptionType.OT_INTEGER
    szDefaultValue = "8080"
    szHelp = "enable Web Server"

    def set(self, value: str) -> None:
        super().set(value)
        if int(value) > 0:
            self.inifile.setaddkeyvalue("UWeb.WebServer", "bEnabled", "True")
        else:
            self.inifile.setaddkeyvalue("UWeb.WebServer", "bEnabled", "False")
        self.inifile.writefile()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

""" Core file of pygamecfg
contain generic management code for GameOption
"""
from __future__ import annotations
from typing import Union

from abc import ABCMeta, abstractmethod
from enum import Enum


class GameCfgException(Exception):
    """Standard Exception to catch non existing key in configuration file"""


class OptionNotFoundError(GameCfgException):
    """Standard Exception to catch non existing configuration option"""


class OptionType(Enum):
    """Supported option data type"""

    OT_INVALID = 0
    OT_STRING = 1
    OT_INTEGER = 2
    OT_BOOLEAN = 3
    OT_FLOAT = 4


class GameOption(metaclass=ABCMeta):
    """Game option base type"""

    szGameType: str = ""
    szOptionName: str = ""
    TValueType: OptionType = OptionType.OT_INVALID
    szDefaultValue: str = ""
    szHelp: str = ""

    def __init__(self, GameRootDir: str, ConfigFileRelPath: Union[None, str] = None):
        """GameOption constructor.

        ///warning
        This object does not aim to be created
        ///

        Args:
            GameRootDir: root dir of the game
            ConfigFileRelPath: path to the configfile (relative to rootdir)
        """
        self.GameRootDir = GameRootDir
        self.ConfigFileRelPath = ConfigFileRelPath

    def __enter__(self) -> GameOption:
        """contextlib enter hook"""
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback) -> None:
        """contextlib exit hook"""
        self.close()

    def close(self) -> None:
        """user-define close() function (for subclassing)"""

    def format_OT_STRING(self, value: Union[int, str, float]) -> str:
        """ "STRING specific format method (TO file)"""
        return str(value)

    def format_OT_INTEGER(self, value: Union[int, str, float]) -> str:
        """ "INTEGER specific format method (TO file)"""
        return str(int(value))

    def format_OT_BOOLEAN(self, value: Union[int, str, float]) -> str:
        """ "BOOLEAN specific format method (TO file)"""
        try:
            intval = int(value)
            return str(bool(intval))
        except ValueError:
            return str(True) if str(value).lower() == "true" else str(False)

    def format_OT_FLOAT(self, value: Union[int, str, float]) -> str:
        """ "FLOAT specific format method (TO file)"""
        return str(float(value))

    def format(self, value: Union[int, str, float]) -> str:
        """standard method to format options before writing it TO file (overloadable)"""

        FormatedValue: str = ""
        if self.TValueType == OptionType.OT_STRING:
            FormatedValue = self.format_OT_STRING(value)
        elif self.TValueType == OptionType.OT_INTEGER:
            FormatedValue = self.format_OT_INTEGER(value)
        elif self.TValueType == OptionType.OT_BOOLEAN:
            FormatedValue = self.format_OT_BOOLEAN(value)
        elif self.TValueType == OptionType.OT_FLOAT:
            FormatedValue = self.format_OT_FLOAT(value)
        else:
            raise RuntimeError("Invalid Option TValueType")

        print(f"setting option <{self.szOptionName}>  to:  {FormatedValue}")
        return FormatedValue

    @abstractmethod
    def set(self, value: str) -> None:
        """generic set function"""
        raise NotImplementedError("method not implemented")

    @abstractmethod
    def rem(self, value: Union[None, str]) -> None:
        """generic rem function"""
        raise NotImplementedError("method not implemented")

    @abstractmethod
    def get(self) -> Union[str, list[str]]:
        """generic get function"""
        raise NotImplementedError("method not implemented")


class GameOptions_Factory:
    """factory that manage game options based on Game and the option itself"""

    ar_Options_cls: list[type[GameOption]] = []
    ar_Options_cls_filtered: list[type[GameOption]] = []
    GameRootDir: str = "./"
    ConfigFileRelPath: str = ""

    def __init__(self, szGameType: str, GameRootDir: str, ConfigFileRelPath: str) -> None:
        self.szGameType = szGameType
        self.GameRootDir = GameRootDir
        self.ConfigFileRelPath = ConfigFileRelPath
        for Options_cls in GameOptions_Factory.ar_Options_cls:
            if Options_cls.szGameType == szGameType:
                self.ar_Options_cls_filtered.append(Options_cls)

    @classmethod
    def GameOptionRegister(cls, Option: type[GameOption]) -> None:
        """interface option used by decorator to register option implementation classes"""
        GameOptions_Factory.ar_Options_cls.append(Option)

    def set(self, OptionName: str, value: str) -> None:
        """generic set function (API call)"""
        for _option in GameOptions_Factory.ar_Options_cls_filtered:
            if _option.szOptionName == OptionName:
                with _option(self.GameRootDir, self.ConfigFileRelPath) as _optionInst:
                    _optionInst.set(_optionInst.format(value))
                return
        raise OptionNotFoundError("Option not found")

    def rem(self, OptionName: str, value: Union[None, str]) -> None:
        """generic rem function (API call)"""
        for _option in self.ar_Options_cls_filtered:
            if _option.szOptionName == OptionName:
                with _option(self.GameRootDir, self.ConfigFileRelPath) as _optionInst:
                    _optionInst.rem(value)
                return
        raise OptionNotFoundError("Option not found")

    def get(self, OptionName: str) -> Union[str, list[str]]:
        """generic get function (API call)"""
        for _option in self.ar_Options_cls_filtered:
            if _option.szOptionName == OptionName:
                with _option(self.GameRootDir, self.ConfigFileRelPath) as _optionInst:
                    return _optionInst.get()
        raise OptionNotFoundError("Option not found")


def GameOptions_Factory_Register(cls: type[GameOption]) -> type[GameOption]:
    """decorator to register game option concrete implementation"""
    GameOptions_Factory.GameOptionRegister(cls)
    return cls



# pygamecfg (c) by chacha
#
# pygamecfg  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

import unittest
from os import chdir
from io import StringIO
from contextlib import redirect_stdout, redirect_stderr
from pathlib import Path

from src import pygamecfg
from src.pygamecfg.__main__ import fct_main


testdir_path = Path(__file__).parent.resolve()


class Testtest_gen(unittest.TestCase):
    def setUp(self) -> None:
        chdir(testdir_path.parent.resolve())

    def test_version(self):
        self.assertNotEqual(pygamecfg.__version__, "?.?.?")

    def test_normal_help(self):
        with self.assertRaises(SystemExit) as cm:
            fct_main(["-h"])

    def test_defect_nogame(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            with self.assertRaises(SystemExit) as cm:
                fct_main(["GetOption", "test"])
            self.assertIn("pygamecfg: error: the following arguments are required: -g/--game", capted_stderr.getvalue())
            self.assertIn("", capted_stdout.getvalue())



# pygamecfg (c) by chacha
#
# pygamecfg  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

import unittest
from os import path, chdir
from io import StringIO
from contextlib import redirect_stdout, redirect_stderr
from pathlib import Path
import shutil

from src import pygamecfg
from src.pygamecfg.__main__ import fct_main

testdir_path = Path(__file__).parent.resolve()


class Testtest_ut99(unittest.TestCase):
    def setUp(self) -> None:
        chdir(testdir_path.parent.resolve())
        self.CleanTmp()
        shutil.copytree(testdir_path / "data", testdir_path / "tmp")
        print("======================")

    def CleanTmp(self):
        # remove any file in tmp dir, except .keep
        if path.exists(testdir_path / "tmp"):
            shutil.rmtree(testdir_path / "tmp")

    def test_normal_ServerPackages(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "ServerPackages"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual(
                "['SoldierSkins', 'CommandoSkins', 'FCommandoSkins', 'SGirlSkins', 'BossSkins', 'Botpack']\n", capted_stdout.getvalue()
            )
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_ServerActors(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "ServerActors"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual(
                "['IpDrv.UdpBeacon', 'IpServer.UdpServerQuery', 'IpServer.UdpServerUplink MasterServerAddress=unreal.epicgames.com MasterServerPort=27900', 'IpServer.UdpServerUplink MasterServerAddress=master0.gamespy.com MasterServerPort=27900', 'IpServer.UdpServerUplink MasterServerAddress=master.mplayer.com MasterServerPort=27900', 'UWeb.WebServer']\n",
                capted_stdout.getvalue(),
            )
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_Port(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "Port"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("7777\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_Map(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "Map"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("Index.unr\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_GameType(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "GameType"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("Botpack.DeathMatchPlus\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_HostName(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "HostName"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("Test Server Name FULL\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MOTD(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MOTD"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestMOTDLine1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MOTD2(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MOTD2"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestMOTDLine2\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MOTD3(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MOTD3"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestMOTDLine3\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MOTD4(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MOTD4"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestMOTDLine4\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AdminEmail(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AdminEmail"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestAdminName@test.com\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AdminName(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AdminName"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestAdminName\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_HTTPDownloadServer(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "HTTPDownloadServer"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("http://uz.ut-files.com/\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MaxClientRate(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MaxClientRate"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("20000\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_NetServerMaxTickRate(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "NetServerMaxTickRate"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("20\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_LanServerMaxTickRate(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "LanServerMaxTickRate"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("35\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AdminPassword(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AdminPassword"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestAdminPwd\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_GamePassword(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "GamePassword"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("TestPwd\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MaxPlayers(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MaxPlayers"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("4\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MaxSpectators(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MaxSpectators"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("1\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AS_TimeLimit(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AS_TimeLimit"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("123\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DOM_TimeLimit(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DOM_TimeLimit"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("423\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_CTF_TimeLimit(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "CTF_TimeLimit"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("223\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DM_TimeLimit(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DM_TimeLimit"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("323\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_GoalTeamScore(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "GoalTeamScore"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("30\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MaxTeamSize(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MaxTeamSize"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("4\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DM_FragLimit(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DM_FragLimit"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("321\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_ServerLogName(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "ServerLogName"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("server.log\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_WebServer(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "WebServer"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("9999\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_TournamentMode(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "TournamentMode"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("False\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_InitialBots(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "InitialBots"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("7\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_MinPlayers(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "MinPlayers"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("11\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AS_UseTranslocator(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AS_UseTranslocator"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("True\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_CTF_UseTranslocator(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "CTF_UseTranslocator"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("True\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DM_UseTranslocator(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DM_UseTranslocator"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("False\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DOM_UseTranslocator(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DOM_UseTranslocator"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("True\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_CTF_ForceRespawn(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "CTF_ForceRespawn"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("False\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DM_ForceRespawn(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DM_ForceRespawn"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("True\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_DOM_ForceRespawn(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "DOM_ForceRespawn"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("False\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_AS_ForceRespawn(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "GetOption", "AS_ForceRespawn"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("True\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())

    def test_normal_customcfgfile_HostName(self):
        with redirect_stdout(StringIO()) as capted_stdout, redirect_stderr(StringIO()) as capted_stderr:
            fct_main(["-g", "ut99", "-b", "test/tmp/UT99", "-c", "System/UT99.ini", "GetOption", "HostName"])
            # /!\ add '\n' at the end of the string cause Python terminal newline is always this, regardless Windows / Linux os.linesep
            self.assertEqual("Alt Serv Name FULL\n", capted_stdout.getvalue())
            self.assertEqual("", capted_stderr.getvalue())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.
# pylint: disable=wrong-import-position
"""
Main module __init__ file.
"""

from importlib.metadata import distribution, version, PackageNotFoundError
import warnings


try:  # pragma: no cover
    __version__ = version("pygamecfg")
except PackageNotFoundError:  # pragma: no cover
    warnings.warn("can not read __version__, assuming local test context, setting it to ?.?.?")
    __version__ = "?.?.?"

try:  # pragma: no cover
    dist = distribution("pygamecfg")
    __Summuary__ = dist.metadata["Summary"]
except PackageNotFoundError:  # pragma: no cover
    warnings.warn('can not read dist.metadata["Summary"], assuming local test context, setting it to <pygamecfg description>')
    __Summuary__ = "pygamecfg description"

try:  # pragma: no cover
    dist = distribution("pygamecfg")
    __Name__ = dist.metadata["Name"]
except PackageNotFoundError:  # pragma: no cover
    warnings.warn('can not read dist.metadata["Name"], assuming local test context, setting it to <pygamecfg>')
    __Name__ = "pygamecfg"

from .core_gamecfg import GameOptions_Factory
from . import game_cod4
from . import game_cod4_gungame
from . import game_cod4_promod
from . import game_ut99
from . import game_ut2k4



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

# pylint: disable=missing-class-docstring,missing-module-docstring,missing-function-docstring,duplicate-code,line-too-long

from __future__ import annotations

from .core_gamecfg import GameOptions_Factory_Register, OptionType
from .game_cod4 import GameOption_COD4


class GameOption_COD4_GunGame(GameOption_COD4):
    szGameType = "cod4_gungame"


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_kills_for_levelup(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "kills_for_levelup"
    szKeyName: str = "gg_kills_for_levelup"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "2"
    szHelp = """Number of kills you need to level-up"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_turbo(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "turbo"
    szKeyName: str = "gg_turbo"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Turbo Mode: Give the new weapon immediately"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_level_down(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "level_down"
    szKeyName: str = "gg_level_down"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Number of levels you lose on suicide, teamkill..."""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_health_regen_time(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "health_regen_time"
    szKeyName: str = "gg_health_regen_time"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "6"
    szHelp = """Time before health regeneration starts: 5 = default, 0 = disable health regen"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_health_text(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "health_text"
    szKeyName: str = "gg_health_text"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Text in the bottom-left part of the screen that shows your health"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_weapon_drop(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "weapon_drop"
    szKeyName: str = "gg_weapon_drop"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Drop weapons on death?"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_knife_nerf(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "knife_nerf"
    szKeyName: str = "gg_knife_nerf"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """0 = default, 1 = knife needs 2 hits to kill"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_dont_cook_frags(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "dont_cook_frags"
    szKeyName: str = "gg_dont_cook_frags"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """1 = don't cook frag grenades, 0 = default nades"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_end_music(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "end_music"
    szKeyName: str = "gg_end_music"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Music at the end of the map"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_knife_pro(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "knife_pro"
    szKeyName: str = "gg_knife_pro"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """When you knife someone you steal a level from him"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_deadly_snipers(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "deadly_snipers"
    szKeyName: str = "gg_deadly_snipers"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """1 = increase damage for bolt-action snipers, 2 = increase damage for all snipers"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_handicap(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "handicap"
    szKeyName: str = "gg_handicap"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """If a player joins in the middle of the game he is auto leveled to the level the worst player has"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_ammo_frag(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "ammo_frag"
    szKeyName: str = "gg_ammo_frag"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "6"
    szHelp = """Ammo for frag grenades [1-10]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_ammo_c4(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "ammo_c4"
    szKeyName: str = "gg_ammo_c4"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "4"
    szHelp = """Ammo for C4 [1-10]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_ammo_rpg(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "ammo_rpg"
    szKeyName: str = "gg_ammo_rpg"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "3"
    szHelp = """Ammo for RPG [1-10]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_rotate_empty(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "rotate_empty"
    szKeyName: str = "gg_rotate_empty"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "10"
    szHelp = """Minutes after which server rotates to next map if no active players"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_remove_turrets(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "remove_turrets"
    szKeyName: str = "gg_remove_turrets"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Remove stationary turrets [0=turrets allwed; 1=turrets removed]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_second_weapon(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "second_weapon"
    szKeyName: str = "gg_second_weapon"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "deserteagle"
    szHelp = """The second weapon the players will have (empty to disable)"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_second_weapon_levels(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "second_weapon_levels"
    szKeyName: str = "gg_second_weapon_levels"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "c4,rpg,frag_grenade,rpg"
    szHelp = """The second weapon the players will have (empty to disable)"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_weapon_sequence(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "weapon_sequence"
    szKeyName: str = "gg_weapon_sequence"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "beretta,usp,colt45,deserteagle,winchester1200,m1014,mp5,m40a3,skorpion,uzi,ak74u,p90,m16,ak47,m4,g3,g36c,m14,mp44,saw,rpd,m60e4,barett,remington700,rpg,frag_grenade,knife"
    szHelp = """comma separated list from: beretta,usp,colt45,deserteagle,winchester1200,m1014,mp5,skorpion,uzi,ak74u,p90,m16,ak47,m4,g3,g36c,m14,mp44,saw,rpd,m60e4,m40a3,m21,dragunov,remington700,barrett,frag_grenade,knife,c4,rpg"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Enable(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "mc_enable"
    szKeyName: str = "mc_enable"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """Enable rotating messages? [0: disable; 1: new custom messages; 2: standard print messages bottom left]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Delay(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "mc_delay"
    szKeyName: str = "mc_delay"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "20"
    szHelp = """Delay between messages (sec)"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_MaxMsg(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_INTEGER
    szOptionName: str = "mc_max_msg"
    szKeyName: str = "mc_max_msg"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "0"
    szHelp = """Maximum number of messages to use [max 20]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_0(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_0"
    szKeyName: str = "mc_msg_0"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 0 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_1(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_1"
    szKeyName: str = "mc_msg_1"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 1 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_2(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_2"
    szKeyName: str = "mc_msg_2"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 2 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_3(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_3"
    szKeyName: str = "mc_msg_3"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 3 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_4(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_4"
    szKeyName: str = "mc_msg_4"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 4 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_5(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_5"
    szKeyName: str = "mc_msg_5"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 5 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_MessageCenter_Message_6(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "mc_msg_6"
    szKeyName: str = "mc_msg_6"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = ""
    szHelp = """message num 6 in rotation"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_WelcomeMessage_Enable(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_BOOLEAN
    szOptionName: str = "wc_enable"
    szKeyName: str = "wc_enable"
    bDblQuoted: bool = False
    szPrefix = "set"
    szDefaultValue = "1"
    szHelp = """Enable welcome message"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_WelcomeMessage_Messsage(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "wc_message"
    szKeyName: str = "wc_message"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = r"Welcome to the server <name>!\Please enjoy your stay!"
    szHelp = r"""The message [ <name> is replaced by players name and \ indicates a new line ]"""


@GameOptions_Factory_Register
class GameOption_COD4_GunGame_bottom_text(GameOption_COD4_GunGame):
    TValueType = OptionType.OT_STRING
    szOptionName: str = "bottom_text"
    szKeyName: str = "gg_bottom_text"
    bDblQuoted: bool = True
    szPrefix = "set"
    szDefaultValue = "GunGame Server"
    szHelp = """ Hud [ custom text displayed on the bottom of the screen next to mod info ]"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pyGameCFG(c) by chacha
#
# pyGameCFG  is licensed under a
# Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Unported License.
#
# You should have received a copy of the license along with this
# work.  If not, see <https://creativecommons.org/licenses/by-nc-sa/4.0/>.

"""CLI interface module"""
from __future__ import annotations
from typing import Literal, cast, Union

import sys
from tap import Tap

from . import __Summuary__, __Name__
from . import GameOptions_Factory


class pygamecfg_args_SetOption(Tap):
    """SetOption CLI arg subparser"""

    option: str
    value: str = ""

    def configure(self) -> None:
        self.add_argument("option")
        self.add_argument("value")


class pygamecfg_args_RemOption(Tap):
    """RemOption CLI arg subparser"""

    option: str
    value: str = ""

    def configure(self) -> None:
        self.add_argument("option")
        self.add_argument("value")


class pygamecfg_args_GetOption(Tap):
    """GetOption CLI arg subparser"""

    option: str

    def configure(self) -> None:
        self.add_argument("option")


class pygamecfg_args(Tap):
    """Main CLI arg parser"""

    verbosity: int = 0
    basegamedir: str = "./"
    configfile: str = ""
    game: Literal["ut99", "cod4", "cod4_gungame", "cod4_promod", "ut2k4"]

    def configure(self) -> None:
        self.add_argument("-v", "--verbosity", action="count", help="increase output verbosity")
        self.add_argument("-b", "--basegamedir", help="set the game base dir")
        self.add_argument("-c", "--configfile", help="set the default config file")
        self.add_argument("-g", "--game", help="the target game")
        self.add_subparsers(dest="command", help="command type", required=True)
        self.add_subparser("SetOption", pygamecfg_args_SetOption, help="Set/Add a game config file option value")
        self.add_subparser("RemOption", pygamecfg_args_RemOption, help="Remove a game config file option, w or w/o value")
        self.add_subparser("GetOption", pygamecfg_args_GetOption, help="Get a game config file option value")

    def process_args(self) -> None:
        """dynamically add self.command to avoid conflict with Tap/argparse while keep pylint happy"""
        self.command: Union[str, None] = cast(Union[str, None], self.command)  # pylint: disable=attribute-defined-outside-init


def fct_main(i_args: list[str]) -> None:
    """CLI main function"""
    parser: pygamecfg_args = pygamecfg_args(prog=__Name__, description=__Summuary__)

    args: pygamecfg_args = parser.parse_args(i_args)

    if args.verbosity:
        print(f"Using base game dir: {args.basegamedir}")
        print(f"Using config file: {args.configfile}")

    if args.configfile == "":
        if args.game == "ut99":
            args.configfile = "./System/UnrealTournament.ini"
        if args.game == "ut2k4":
            args.configfile = "./System/UT2004.ini"
        elif args.game in ("cod4", "cod4_gungame", "cod4_promod"):
            args.configfile = "./main/server.cfg"

    GameOptions = GameOptions_Factory(args.game, args.basegamedir, args.configfile)
    if args.command == "SetOption":
        GameOptions.set(
            cast(pygamecfg_args_SetOption, args).option, cast(pygamecfg_args_SetOption, args).value  # pylint: disable=no-member
        )
    elif args.command == "RemOption":
        GameOptions.rem(
            cast(pygamecfg_args_RemOption, args).option, cast(pygamecfg_args_RemOption, args).value  # pylint: disable=no-member
        )
    elif args.command == "GetOption":
        res = GameOptions.get(cast(pygamecfg_args_GetOption, args).option)  # pylint: disable=no-member
        print(res)
    else:
        raise RuntimeError("Invalid argument")


if __name__ == "__main__":
    fct_main(sys.argv[1:])
